(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
    typeof define === 'function' && define.amd ? define(['exports'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.WebMessengerWorkerMain = {}));
})(this, (function (exports) { 'use strict';

    /******************************************************************************
    Copyright (c) Microsoft Corporation.

    Permission to use, copy, modify, and/or distribute this software for any
    purpose with or without fee is hereby granted.

    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
    PERFORMANCE OF THIS SOFTWARE.
    ***************************************************************************** */
    /* global Reflect, Promise, SuppressedError, Symbol */


    function __awaiter(thisArg, _arguments, P, generator) {
        function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
        return new (P || (P = Promise))(function (resolve, reject) {
            function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
            function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
            function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
            step((generator = generator.apply(thisArg, _arguments || [])).next());
        });
    }

    typeof SuppressedError === "function" ? SuppressedError : function (error, suppressed, message) {
        var e = new Error(message);
        return e.name = "SuppressedError", e.error = error, e.suppressed = suppressed, e;
    };

    var SCENE_TYPE;
    (function (SCENE_TYPE) {
        SCENE_TYPE["IFRAME"] = "iframe";
        SCENE_TYPE["WORKER"] = "worker";
    })(SCENE_TYPE || (SCENE_TYPE = {}));
    var MESSAGE_TYPE;
    (function (MESSAGE_TYPE) {
        MESSAGE_TYPE["REQUEST"] = "request";
        MESSAGE_TYPE["RESPONSE"] = "response";
    })(MESSAGE_TYPE || (MESSAGE_TYPE = {}));

    const iframeMessageUnresolvedPool = {};
    const workerMessageUnresolvedPool = {};
    function getPool(type) {
        switch (type) {
            case SCENE_TYPE.IFRAME:
                return iframeMessageUnresolvedPool;
            case SCENE_TYPE.WORKER:
                return workerMessageUnresolvedPool;
            default:
                throw new Error("type is not supported");
        }
    }
    function saveMessageUnresolvedPool(type, messageId, resolve) {
        const pool = getPool(type);
        pool[messageId] = resolve;
    }
    function resolveResponseMessage(type, messageData) {
        const { messageId, data } = messageData;
        const pool = getPool(type);
        const resolve = pool[messageId];
        if (resolve) {
            resolve(data);
            pool[messageId] = null;
        }
    }

    function useId() {
        const id = `${Date.now().toString()}_${Math.random().toFixed(4)}`;
        return id;
    }

    let _worker = null;
    function setWorker(worker) {
        _worker = worker;
    }
    function clearWorker() {
        _worker = null;
    }
    function postMessageToWorker$1({ messageType, messageId, data, transfer, callName, }) {
        if (!_worker) {
            throw new Error("worker is not set");
        }
        const message = {
            messageType,
            callName,
            messageId,
            data,
        };
        if (transfer) {
            _worker.postMessage(message, { transfer });
            return;
        }
        _worker.postMessage(message);
    }
    function postRequestToWorker({ callName, data, transfer, awaitResponse = true, }) {
        return __awaiter(this, void 0, void 0, function* () {
            const messageId = useId();
            postMessageToWorker$1({
                messageType: MESSAGE_TYPE.REQUEST,
                messageId,
                data,
                transfer,
                callName,
            });
            if (!awaitResponse)
                return;
            return new Promise((resolve) => {
                saveMessageUnresolvedPool(SCENE_TYPE.WORKER, messageId, resolve);
            });
        });
    }
    function postResponseToWorker({ messageId, data, transfer, }) {
        postMessageToWorker$1({
            messageType: MESSAGE_TYPE.RESPONSE,
            messageId,
            data,
            transfer,
        });
    }

    function getLogName(key) {
        return `[__${key}__]`;
    }
    const KEY = "WEB_MESSENGER";
    function commonLogName() {
        return getLogName(KEY);
    }
    function genCommonLogMessage(message) {
        return `${commonLogName()} ${message}`;
    }
    function errorMessage(message) {
        return new Error(genCommonLogMessage(message));
    }

    function commonMessageHandler({ messageDispatcher, event, postResponse, receiveResponse, }) {
        return __awaiter(this, void 0, void 0, function* () {
            const { messageType, messageId, callName, data } = event.data;
            switch (messageType) {
                case MESSAGE_TYPE.REQUEST: {
                    if (!callName) {
                        throw errorMessage("callName is required");
                    }
                    if (!messageDispatcher) {
                        throw errorMessage("messageDispatcher not found");
                    }
                    if (!messageDispatcher[callName] ||
                        typeof messageDispatcher[callName] !== "function") {
                        throw errorMessage(`messageDispatcher.${callName} neither found or callable`);
                    }
                    if (callName.endsWith("Transferable")) {
                        const result = yield messageDispatcher[callName](data);
                        if (!Array.isArray(result)) {
                            throw errorMessage(`messageDispatcher.${callName} transferable must return tuple [data, transfer]`);
                        }
                        const [res, transfer] = result;
                        postResponse(messageId, res, transfer);
                        break;
                    }
                    const res = yield messageDispatcher[callName](data);
                    postResponse(messageId, res);
                    break;
                }
                case MESSAGE_TYPE.RESPONSE:
                    receiveResponse(messageId, data);
                    break;
            }
        });
    }

    function registerMain(messageDispatcher, options) {
        if (!(options === null || options === void 0 ? void 0 : options.worker)) {
            throw errorMessage("options.worker is required");
        }
        setWorker(options.worker);
        function messageHandler(event) {
            return __awaiter(this, void 0, void 0, function* () {
                commonMessageHandler({
                    messageDispatcher,
                    event,
                    postResponse: (messageId, data, transfer) => {
                        postResponseToWorker({
                            messageId,
                            data,
                            transfer,
                        });
                    },
                    receiveResponse: (messageId, data) => {
                        resolveResponseMessage(SCENE_TYPE.WORKER, { messageId, data });
                    },
                });
            });
        }
        options.worker.addEventListener("message", messageHandler);
        const cleanup = () => {
            options.worker.removeEventListener("message", messageHandler);
        };
        return {
            postToWorker(callName, data) {
                return postRequestToWorker({
                    callName,
                    data,
                    awaitResponse: false,
                });
            },
            postToWorkerAwaitResponse(callName, data) {
                return postRequestToWorker({
                    callName,
                    data,
                });
            },
            postToWorkerTransferable(callName, data, transfer) {
                return postRequestToWorker({
                    callName,
                    data,
                    transfer,
                    awaitResponse: false,
                });
            },
            postToWorkerTransferableAwaitResponse(callName, data, transfer) {
                return postRequestToWorker({
                    callName,
                    data,
                    transfer,
                });
            },
            cleanup() {
                cleanup();
                clearWorker();
            },
        };
    }
    const postMessageToWorker = postRequestToWorker;

    exports.postMessageToWorker = postMessageToWorker;
    exports.registerMain = registerMain;

}));
//# sourceMappingURL=webMessenger-workerMain.umd.js.map
